package ru.dbotthepony.kstarbound.io.json

import com.google.common.collect.ImmutableList
import com.google.gson.JsonSyntaxException
import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kstarbound.Starbound

class ListAdapter<T>(private val type: Class<T>) : TypeAdapter<List<T>?>() {
	private val resolvedBound by lazy { Starbound.getTypeAdapter(type) }

	override fun write(out: JsonWriter, value: List<T>?) {
		out.beginArray()

		if (value != null) {
			val resolvedBound = resolvedBound

			for (v in value) {
				resolvedBound.write(out, v)
			}
		}

		out.endArray()
	}

	override fun read(reader: JsonReader): List<T> {
		reader.beginArray()

		val builder = ImmutableList.builder<T>()
		val resolvedBound = resolvedBound

		while (reader.peek() != JsonToken.END_ARRAY) {
			val readObject = resolvedBound.read(reader) ?: throw JsonSyntaxException("List does not accept nulls")
			builder.add(readObject as T)
		}

		reader.endArray()

		return builder.build()
	}
}