package ru.dbotthepony.kstarbound.io.json

import com.google.common.collect.ImmutableMap
import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kstarbound.Starbound

class MapAdapter<K, V>(private val keyType: Class<K>, private val valueType: Class<V>) : TypeAdapter<Map<K, V>?>() {
	private val resolvedKey by lazy { Starbound.getTypeAdapter(keyType) }
	private val resolvedValue by lazy { Starbound.getTypeAdapter(valueType) }

	override fun write(out: JsonWriter, value: Map<K, V>?) {
		out.beginArray()

		if (value != null) {
			val resolvedKey = resolvedKey
			val resolvedValue = resolvedValue

			for ((k, v) in value) {
				out.beginArray()
				resolvedKey.write(out, k)
				resolvedValue.write(out, v)
				out.endArray()
			}
		}

		out.endArray()
	}

	override fun read(reader: JsonReader): Map<K, V> {
		reader.beginArray()

		val builder = ImmutableMap.builder<K, V>()

		val resolvedKey = resolvedKey
		val resolvedValue = resolvedValue

		while (reader.peek() != JsonToken.END_ARRAY) {
			reader.beginArray()
			builder.put(resolvedKey.read(reader), resolvedValue.read(reader))
			reader.endArray()
		}

		reader.endArray()

		return builder.build()
	}
}