package ru.dbotthepony.kstarbound.io.json

import com.google.common.collect.ImmutableMap
import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kstarbound.Starbound

class StringMapAdapter<V>(private val type: Class<V>) : TypeAdapter<Map<String, V>?>() {
	private val resolvedBound by lazy { Starbound.getTypeAdapter(type) }

	override fun write(out: JsonWriter, value: Map<String, V>?) {
		val resolvedBound = resolvedBound

		out.beginObject()

		if (value != null) {
			for ((k, v) in value) {
				out.name(k)
				resolvedBound.write(out, v)
			}
		}

		out.endObject()
	}

	override fun read(reader: JsonReader): Map<String, V> {
		val builder = ImmutableMap.builder<String, V>()

		reader.beginObject()

		val resolvedBound = resolvedBound

		while (reader.peek() != JsonToken.END_OBJECT) {
			builder.put(Starbound.assetStringInterner.intern(reader.nextName()), resolvedBound.read(reader))
		}

		reader.endObject()

		return builder.build()
	}
}